<?php
App::uses('LauncherAppController', 'Launcher.Controller');
class LaunchersController extends LauncherAppController
{
    /**
     * クラス名
     *
     * @var string
     * @access public
     */
    public $name = 'Launchers';

    /**
     * コンポーネント
     *
     * @var array
     * @access public
     */
    public $components = array('BcAuth', 'Cookie', 'BcAuthConfigure', 'Security', 'BcContents');

    /**
     * モデル
     *
     * @var array
     * @access public
     */
    public $uses = array('Launcher.Launcher');

    /**
     * ぱんくずナビ
     *
     * @var string
     * @access public
     */
    public $crumbs = array(
        array('name' => 'プラグイン管理', 'url' => array('plugin' => '', 'controller' => 'plugins', 'action' => 'index')),
        array('name' => 'ランチャー管理', 'url' => array('controller' => 'launchers', 'action' => 'index'))
    );

    /**
     * サブメニューエレメント
     *
     * @var array
     * @access public
     */
    public $subMenuElements = array('launchers');

    /**
     * [ADMIN] 一覧表示
     *
     * @return void
     */
    public function admin_index()
    {
    // 画面表示情報設定
    $default = array(
        'named' => array(
            'num' => $this->siteConfigs['admin_list_num']
        ),
    );
    $this->setViewConditions('Launcher', array('default' => $default));

    // 並び替えモードの場合は、強制的にsortフィールドで並び替える
    if (!empty($this->passedArgs['sortmode'])) {
        $this->passedArgs['sort'] = 'sort';
        $this->passedArgs['direction'] = 'asc';
        $this->set('sortmode', true);
        //設定
        $this->paginate = array(
            'fields' => array(),
            'order' => 'Launcher.sort',
            'limit' => $this->passedArgs['num']
        );
    } else {
        $this->set('sortmode', false);
        //設定
        $this->paginate = array(
            'fields' => array(),
            'order' => 'Launcher.id DESC',
            'limit' => $this->passedArgs['num']
        );
    }

    /* 表示設定 */
    $this->set('launchers', $this->paginate('Launcher'));
    $this->pageTitle = 'ランチャーアイコン一覧';
    }

    /**
     * [ADMIN] 新規登録
     *
     * @return void
     */
    public function admin_add($id = null)
    {
    //登録ボタン押下後
    if (!empty($this->request->data)) {

        //画像保存処理実行
        $insertData = $this->Launcher->savePostImg($this->request->data);

        //現在の最大ソート番号を取得する。
        $maxSort = $this->Launcher->find('first', array(
            'fields' => "MAX(Launcher.sort) as max_num"));

        //次回のソート番号
        $nextCount = $maxSort[0]['max_num'] +1;
        $insertData['Launcher']['sort'] = $nextCount;

        //生成
        $this->Launcher->create();

        //保存
        if ($this->Launcher->save($insertData)) {
            $this->setMessage('ランチャーアイコン「'. h($this->request->data['Launcher']['title']).'」を登録しました。', false, true);
            $this->redirect(array('action' => 'index'));
        } else {
            $this->setMessage('入力エラーです。内容を修正してください。', true);
        }
    }

    /* 表示設定 */
    $this->pageTitle = 'ランチャーアイコン 新規登録';
    $this->render('form');
    }

    /**
     * [ADMIN] 編集
     *
     * @return void
     */
    public function admin_edit($id = null)
    {
        /* 除外処理 */
        if (empty($id)) {
            $this->setMessage('無効なIDです。', true);
            $this->redirect(array('action' => 'index'));
        }

        /* 既存データ取得 */
        $Launcher = $this->Launcher->findById($id);

        /* 除外処理 */
        if (empty($Launcher)) {
            $this->setMessage('無効なIDです。', true);
            $this->redirect(array('action' => 'index'));
        }

        //登録ボタン押下後
        if (!empty($this->request->data)) {
            //画像保存処理実行
            $insertData = $this->Launcher->savePostImg($this->request->data);

            //保存
            if ($this->Launcher->save($insertData)) {
                $this->setMessage('ランチャーアイコン「'. h($this->request->data['Launcher']['title']).'」を編集しました。', false, true);
                $this->redirect(array('action' => 'index'));
            } else {
                $this->setMessage('入力エラーです。内容を修正してください。', true);
            }

            //編集画面初期表示
        } else {
            $this->request->data = $Launcher;
        }

        //画像があれば表示させるため、取得した情報と画像パスをセットする。
        $this->set('Launcher', $Launcher);
        $this->set('imgPath', '/img/Launcher_plugin/');

        /* 表示設定 */
        $this->pageTitle = 'ランチャーアイコン 編集';
        $this->render('form');
    }

    /**
     * [ADMIN] 削除
     *
     * @return void
     */
    public function admin_delete($id = null)
    {
    /* 除外処理 */
    if (empty($id)) {
        $this->setMessage('無効なIDです。', true);
        $this->redirect(array('action' => 'index'));
    }

    //情報の取得
    $Launcher = $this->Launcher->findById($id);

    if ($this->Launcher->delete($id)) {
        //画像も削除する。
        if (!empty($Launcher['Launcher']['image_1'])) {
            $this->Launcher->deleteSaveImage($Launcher['Launcher']['image_1']);
        }
        $this->setMessage('ランチャーアイコン「'.h($Launcher['Launcher']['title']).'」を削除しました。', false, true);
    } else {
        $this->setMessage('入力エラーです。削除に失敗しました。', true);
    }

    /* 表示設定 */
    $this->redirect(array('action' => 'index'));
    }

    /**
     * [ADMIN] 利用状態変更
     *
     * @return void
     */
    public function admin_publish($id = null)
    {
    /* 除外処理 */
    if (empty($id)){
        $this->setMessage('無効なIDです。', true);
        $this->redirect(array('action' => 'index'));
    }
    //現在の情報を取得する。
    $data = $this->Launcher->find('first', array('conditions' => array(
        'Launcher.id' => $id
    )));
    //非公開設定
    switch($data['Launcher']['publish']){
        case 1:
            $data['Launcher']['publish'] = 0;
            break;
        case 0:
            $data['Launcher']['publish'] = 1;
            break;
    }

    //挿入データの整形
    $insert['Launcher'] = $data['Launcher'];

    //更新処理実行
    $this->Launcher->id = $id;
    if ($this->Launcher->save($insert)) {
        $this->setMessage('ランチャーアイコン「'. h($data['Launcher']['title']). '」の公開設定を切替ました。', false, true);
        clearViewCache();
        clearDataCache();
        $this->redirect(array('action' => 'admin_index'));
        exit(true);
        // return true;
    } else {
        //$this->setMessage('入力エラーです。内容を修正してください。', true);
        $this->redirect(array('action' => 'admin_index'));
        //return false;
        exit(false);
    }
    }

    /**
     * 並び替えを更新する [AJAX]
     *
     * @access public
     * @return boolean
     */
    public function admin_ajax_update_sort() 
    {
    if ($this->request->data) {
        $this->setViewConditions('Launcher', array('action' => 'admin_index'));
        $conditions = array();
        if ($this->Launcher->changeSort($this->request->data['Sort']['id'], $this->request->data['Sort']['offset'], $conditions)) {
            clearViewCache();
            clearDataCache();
            echo true;
        } else {
            //  debug($this->Launcher->validationErrors);exit;
            $this->ajaxError(500, '一度リロードしてから再実行してみてください。');
        }
    } else {
        $this->ajaxError(500, '無効な処理です。');
    }
    exit();
    }
}
