<?php
App::uses('LauncherAppModel', 'Launcher.Model');
App::uses('Imageresizer', 'Vendor'); //標準搭載の画像処理クラス

class Launcher extends LauncherAppModel {

    /**
     * クラス名
     *
     * @var string
     */
    public $name = 'Launcher';

    /**
     * 画像保存パス
     *
     * @var string
     * @access public
     */
    public $imageDir = 'launcher_plugin';

    /**
     * 画像のデフォルト幅（px）
     *
     * @var string
     * @access public
     */
    public $widthSize = 500;

    /**
     * 画像のデフォルト高さ（px）
     *
     * @var string
     * @access public
     */
    public $heightSize = 500;

    /**
     * 画像の枚数
     *
     * @var string
     * @access public
     */
    public $imageCount = 1;

    /**
     * validate
     *
     * @var array
     */
    public $validate = array(
        'title' => array(
            'notBlank' => array(
                'rule' => array('notBlank'),
                'message' => '名称を入力してください。'
            )),
        'url' => array(
            'notBlank' => array(
                'rule' => array('notBlank'),
                'message' => 'URLを入力してください。'
            )),
        'image_1' => array(
            'notBlank' => array(
                'rule' => array('notBlank'),
                'message' => '画像を入力してください。'
            ))
    );

    /**
     * 画像を保存する。
     *
     * @params array $data
     * @params init $width
     * @params init $height
     * @return bool
     * @access public
     */
    public function savePostImg($data, $width = null, $height = null){

        //現在のデータを取得する。（画像に変更がなかった場合の処理）
        if (!empty($data['Launcher']['id'])) {
            $LauncherData = $this->find('first', array('conditions' => array(
                'Launcher.id' => $data['Launcher']['id']
            )));
        }

        //画像の幅指定
        if (!empty($width)) {
            $limitWidth = $width;
        } else {
            $limitWidth = $this->widthSize; //デフォルト
        }
        if (!empty($height)) {
            $limitHeight = $height;
        } else {
            $limitHeight = $this->heightSize; //デフォルト
        }

        //ファイル送信の確認と保存処理
        for ($i = 1; $i <= $this->imageCount; $i++) {
            if (empty($data['Launcher']['del_image_'.$i]) || $data['Launcher']['del_image_'.$i] == 0) { //削除指定がない場合にのみアップ作業
                if (!empty($data['Launcher']['image_'.$i]['tmp_name'])) {
                    //現在のファイル名と、そこから拡張子を確認する。
                    $img = $this->_copyTmpImage($data['Launcher']['image_'.$i], 'Launcher_'.$i.'_' , $limitWidth, $limitHeight);
                    $data['Launcher']['image_'.$i] = $img;

                    //ファイル送信なしの場合
                } else {
                    //旧画像名をそのままセットする。
                    if (!empty($LauncherData['Launcher']['image_'.$i])) {
                        $data['Launcher']['image_'.$i] = $LauncherData['Launcher']['image_'.$i];
                    } else {
                        $data['Launcher']['image_'.$i] = "";
                    }
                }

                //既存のファイルの削除指示がある場合。
            } else {
                $this->_deletePostImage('image_'.$i, $data);
                $data['Launcher']['image_'.$i] = ''; //フィールドも空をセット
            }
        }
        return $data;
    }

    /**
     * 画像を削除する。
     *
     * @params string $imgKey
     * @params array $data
     * @return bool
     * @access public
     */
    public function deleteSaveImage($imageName){
        //削除処理
        if (!empty($imageName)) {
            $delFileName = WWW_ROOT . 'img' . DS . $this->imageDir . DS . $imageName;
            if (file_exists($delFileName)) {
                @unlink($delFileName);
                return true;
            }else{
                return false;
            }
        }
    }

    /**
     * 画像データだけの配列を返す
     *
     * @params int $categoryId
     * @return array
     * @access public
     */
    public function getImageGallary($categoryId = null){
        //画像ギャラリー格納用
        $gallary = array();
        //検索条件
        $conditions = array();
        //カテゴリ有無
        if (!empty($categoryId)) {
            $conditions[] = array('Launcher.Launcher_category_id' => $categoryId);
        }
        //必須条件
        $conditions[] = array(
            'Launcher.publish' => 1,
            'AND' => array('OR' => array(
                array('LauncherCategory.publish' => 1),
                array('Launcher.Launcher_category_id' => null)
            )));
        //データの取得
        $LauncherData = $this->find('all', array('conditions' => $conditions));
        //取得した画像を整理する。
        foreach ($LauncherData as $data) {
            $images = array(); //一時的に画像データとidを保存する。
            if (!empty($data['Launcher']['image_1'])) {
                $images['image'] = $data['Launcher']['image_1'];
                $images['id'] = $data['Launcher']['id'];
                $gallary[] = $images;
            }
            if (!empty($data['Launcher']['image_2'])) {
                $images['image'] = $data['Launcher']['image_2'];
                $images['id'] = $data['Launcher']['id'];
                $gallary[] = $images;
            }
            if (!empty($data['Launcher']['image_3'])) {
                $images['image'] = $data['Launcher']['image_3'];
                $images['id'] = $data['Launcher']['id'];
                $gallary[] = $images;
            }
            if (!empty($data['Launcher']['image_4'])) {
                $images['image'] = $data['Launcher']['image_4'];
                $images['id'] = $data['Launcher']['id'];
                $gallary[] = $images;
            }
        }
        //結果を返す
        return $gallary; //連想配列で[image][id]を持つ。
    }

    /**
     * 一時ファイルから公開フォルダ上へコピーして
     * 新しいファイル名を返す。
     *
     * @params array $data
     * @params string $prefix
     * @params int $width
     * @params int $height
     * @return string
     * @access protected
     */
    protected function _copyTmpImage($data, $prefix, $width, $height) {
        //コピー元
        $fileName = $data['name'];
        $ext = pathinfo($fileName, PATHINFO_EXTENSION);
        //新しいファイル名を生成
        $fileNamePrefix = $prefix;
        $fileNameNo = date("YmdHis");
        $newFileName =  $fileNamePrefix.$fileNameNo;
        $imgPath = WWW_ROOT . 'img' . DS . $this->imageDir . DS . $newFileName . '.' . $ext;
        //一時ファイルから動かす。
        //サムネイル生成
        $Imageresizer = new Imageresizer();
        $Imageresizer->resize($data['tmp_name'], $imgPath, $width, $height);
        //一時ファイルを消し去ってしまう。
        return $newFileName . '.' . $ext;
    }

    /**
     * 画像を削除する。
     *
     * @params string $imgKey
     * @params array $data
     * @return bool
     * @access protected
     */
    protected function _deletePostImage($imgKey, $data){

        //現在のデータを確認する。
        $LauncherData = $this->find('first', array(
            'conditions' => array(
                'Launcher.id' => $data['Launcher']['id']
            ),
            'field' => array(
                "Launcher.$imgKey" //メモリ節約の為必要なフィールドのみ取得
            )));
        //削除処理
        if (!empty($LauncherData['Launcher']["$imgKey"])) {
            $delFileName = WWW_ROOT . 'img' . DS . $this->imageDir . DS . $LauncherData['Launcher']["$imgKey"];
            if (file_exists($delFileName)) {
                @unlink($delFileName);
                return true;
            }else{
                return false;
            }
        }
    }

    /**
     * 並び順を変更する
     * @param string	$id
     * @param int			$offset
     * @param array		$conditions
     * @return boolean
     */
    public function changeSort($id, $offset, $conditions = array()) {

        //バリデーション解除
        unset($this->validate);

        if ($conditions) {
            $_conditions = $conditions;
        } else {
            $_conditions = array();
        }

        // 一時的にキャッシュをOFFする
        $this->cacheQueries = false;

        $current = $this->find('first', array(
            'conditions' => array($this->alias . '.id' => $id),
            'fields' => array($this->alias . '.id', $this->alias . '.sort')
        ));

        // 変更相手のデータを取得
        if ($offset > 0) { // DOWN
            $order = array($this->alias . '.sort');
            $limit = $offset;
            $conditions[$this->alias . '.sort >'] = $current[$this->alias]['sort'];
        } elseif ($offset < 0) { // UP
            $order = array($this->alias . '.sort DESC');
            $limit = $offset * -1;
            $conditions[$this->alias . '.sort <'] = $current[$this->alias]['sort'];
        } else {
            return true;
        }

        $conditions = array_merge($conditions, $_conditions);
        $target = $this->find('all', array(
            'conditions' => $conditions,
            'fields' => array($this->alias . '.id', $this->alias . '.sort'),
            'order' => $order,
            'limit' => $limit,
            'recursive' => -1
        ));

        if (!isset($target[count($target) - 1])) {
            //ここで落ちる
            return false;
        }

        $currentSort = $current[$this->alias]['sort'];
        $targetSort = $target[count($target) - 1][$this->alias]['sort'];

        // current から target までのデータをsortで範囲指定して取得
        $conditions = array();
        if ($offset > 0) { // DOWN
            $conditions[$this->alias . '.sort >='] = $currentSort;
            $conditions[$this->alias . '.sort <='] = $targetSort;
        } elseif ($offset < 0) { // UP
            $conditions[$this->alias . '.sort <='] = $currentSort;
            $conditions[$this->alias . '.sort >='] = $targetSort;
        }

        $conditions = array_merge($conditions, $_conditions);
        $datas = $this->find('all', array(
            'conditions' => $conditions,
            'fields' => array($this->alias . '.id', $this->alias . '.sort'),
            'order' => $order,
            'recursive' => -1
        ));

        // 全てのデータを更新
        foreach ($datas as $data) {
            if ($data[$this->alias]['sort'] == $currentSort) {
                $data[$this->alias]['sort'] = $targetSort;
            } else {
                if ($offset > 0) {
                    $data[$this->alias]['sort'] --;
                } elseif ($offset < 0) {
                    $data[$this->alias]['sort'] ++;
                }
            }
            if (!$this->save($data, false)) {
                return false;
            }
        }
        return true;
    }

}
